==> hardware-monitor-0.1.tar.gz: hardware-monitor-0.1/src/bar-view.cpp <==
/* Implementation of the bar view.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <config.h>

#include <libgnomecanvasmm/rect.h>
#include <gconfmm/client.h>

#include "ucompose.hpp"

#include "bar-view.hpp"
#include "applet.hpp"
#include "monitor.hpp"

#include "i18n.hpp"


//
// class Bar - represents a single bar graph
//

class Bar
{
public:
  Bar(Monitor *monitor, const Glib::ustring &mon_dir);

  void update(Gnome::Canvas::Canvas &canvas,
	      const Glib::RefPtr<Gnome::Conf::Client> &client,
	      int width, int height, int no, int total); 

  Monitor *get_monitor();
  Glib::ustring get_tip();
  
private:
  Monitor *monitor;
  std::auto_ptr<Gnome::Canvas::Rect> bar;

  double value;
  
  Glib::ustring monitor_dir;
};

Bar::Bar(Monitor *m, const Glib::ustring &mon_dir)
  : monitor(m), value(0), monitor_dir(mon_dir)
{}

void Bar::update(Gnome::Canvas::Canvas &canvas,
	       const Glib::RefPtr<Gnome::Conf::Client> &client,
	       int width, int height, int no, int total)
{
  // make sure bar is initialised
  if (bar.get() == 0)
    bar.reset(new Gnome::Canvas::Rect(*canvas.root()));

  // get drawing attributes
  unsigned int color;

  Gnome::Conf::Value v;
  
  v = client->get(monitor_dir + "/color");
  if (v.get_type() == Gnome::Conf::VALUE_INT)
    color = v.get_int();
  else {
    // FIXME: make up changing colors
    color = 0x0000FF7F;		// nice blue color
    client->set(monitor_dir + "/color", int(color));
  }

  bar->property_fill_color_rgba() = color;


  // draw
  value = monitor->value();
  
  double max = monitor->max();
  if (max <= 0)
    max = 0.0000001;

  // start from left (FIXME: reverse for some locales?)
  bar->property_x1() = 0;
  bar->property_x2() = width * value / max;

  bar->property_y1() = double(height) * no / total;
  bar->property_y2() = double(height) * (no + 1) / total;
}

Monitor *Bar::get_monitor()
{
  return monitor;
}

Glib::ustring Bar::get_tip()
{
  // note to translators: %1 is the name of a monitor, e.g. "CPU 1", and %2 is
  // the current measurment, e.g. "78%"
  return String::ucompose(_("%1: %2"), monitor->get_short_name(),
			  monitor->format_value(value));
}


//
// class BarView
//

BarView::BarView()
{
}

BarView::~BarView()
{
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i)
    delete *i;
}


void BarView::do_display(Applet &a)
{
  applet = &a;
  
  // canvas creation magic
  canvas.reset(new Gnome::Canvas::Canvas);
  canvas->property_aa() = true;
  a.get_container().add(*canvas);
}

void BarView::do_update()
{
  // first update the configured attributes
  Glib::RefPtr<Gnome::Conf::Client> &client = applet->get_gconf_client();
  Glib::ustring dir = applet->get_gconf_dir();

  // FIXME: use schemas?
  if (client->get(dir + "/viewer/size").get_type() == Gnome::Conf::VALUE_INT)
    size = client->get_int(dir + "/viewer/size");
  else {
    size = 60;
    client->set(dir + "/viewer/size", size);
  }

  // ensure the canvas is shown
  resize_canvas();
  canvas->show();

  int total = bars.size();
  int no = 0;
  
  // then loop through each bar
  Glib::ustring tip;
	
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i) {
    Bar &bar = **i;

    if (tip.empty())
      tip = bar.get_tip();
    else
      // note to translators: this is used for composing a list of monitors; %1
      // is the previous part of the list and %2 is the part to append
      tip = String::ucompose(_("%1\n%2"), tip, bar.get_tip());

    bar.update(*canvas, client, width(), height(), no++, total);
  }
  
  tooltips.set_tip(*canvas, tip);
}

void BarView::do_attach(Monitor *monitor, const Glib::ustring &mon_dir)
{
  bars.push_back(new Bar(monitor, mon_dir));
}

void BarView::do_detach(Monitor *monitor)
{
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i)
    if ((*i)->get_monitor() == monitor) {
      delete *i;
      bars.erase(i);
      return;
    }

  g_assert_not_reached();
}

int BarView::width() const
{
  if (applet->horizontal())
    return size;
  else
    return applet->get_size();
}

int BarView::height() const
{
  if (applet->horizontal())
    return applet->get_size();
  else
    return size;
}

void BarView::resize_canvas()
{
  int w = width(), h = height();
  
  canvas->set_scroll_region(0, 0, w, h);
  canvas->set_size_request(w, h);
}

==> hardware-monitor-0.2.tar.gz: hardware-monitor-0.2/src/bar-view.cpp <==
/* Implementation of the bar view.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <config.h>

#include <libgnomecanvasmm/rect.h>
#include <gconfmm/client.h>

#include "ucompose.hpp"

#include "bar-view.hpp"
#include "applet.hpp"
#include "monitor.hpp"

#include "i18n.hpp"


//
// class Bar - represents a single bar graph
//

class Bar
{
public:
  Bar(Monitor *monitor, const Glib::ustring &mon_dir);

  void update(Gnome::Canvas::Canvas &canvas,
	      const Glib::RefPtr<Gnome::Conf::Client> &client,
	      int width, int height, int no, int total); 

  Monitor *get_monitor();
  Glib::ustring get_tip();
  
private:
  Monitor *monitor;
  std::auto_ptr<Gnome::Canvas::Rect> bar;

  double value;
  
  Glib::ustring monitor_dir;
};

Bar::Bar(Monitor *m, const Glib::ustring &mon_dir)
  : monitor(m), value(0), monitor_dir(mon_dir)
{}

void Bar::update(Gnome::Canvas::Canvas &canvas,
	       const Glib::RefPtr<Gnome::Conf::Client> &client,
	       int width, int height, int no, int total)
{
  // make sure bar is initialised
  if (bar.get() == 0)
    bar.reset(new Gnome::Canvas::Rect(*canvas.root()));

  // get drawing attributes
  unsigned int color;

  Gnome::Conf::Value v;
  
  v = client->get(monitor_dir + "/color");
  if (v.get_type() == Gnome::Conf::VALUE_INT)
    color = v.get_int();
  else {
    // FIXME: make up changing colors
    color = 0x0000FF7F;		// nice blue color
    client->set(monitor_dir + "/color", int(color));
  }

  bar->property_fill_color_rgba() = color;


  // draw
  value = monitor->value();
  
  double max = monitor->max();
  if (max <= 0)
    max = 0.0000001;

  // start from left (FIXME: reverse for some locales?)
  bar->property_x1() = 0;
  bar->property_x2() = width * value / max;

  bar->property_y1() = double(height) * no / total;
  bar->property_y2() = double(height) * (no + 1) / total;
}

Monitor *Bar::get_monitor()
{
  return monitor;
}

Glib::ustring Bar::get_tip()
{
  // note to translators: %1 is the name of a monitor, e.g. "CPU 1", and %2 is
  // the current measurement, e.g. "78%"
  return String::ucompose(_("%1: %2"), monitor->get_short_name(),
			  monitor->format_value(value));
}


//
// class BarView
//

BarView::BarView()
{
}

BarView::~BarView()
{
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i)
    delete *i;
}


void BarView::do_display(Applet &a)
{
  applet = &a;
  
  // canvas creation magic
  canvas.reset(new Gnome::Canvas::Canvas);
  canvas->property_aa() = true;
  a.get_container().add(*canvas);
}

void BarView::do_update()
{
  // first update the configured attributes
  Glib::RefPtr<Gnome::Conf::Client> &client = applet->get_gconf_client();
  Glib::ustring dir = applet->get_gconf_dir();

  // FIXME: use schemas?
  if (client->get(dir + "/viewer/size").get_type() == Gnome::Conf::VALUE_INT)
    size = client->get_int(dir + "/viewer/size");
  else {
    size = 60;
    client->set(dir + "/viewer/size", size);
  }

  // ensure the canvas is shown
  resize_canvas();
  canvas->show();

  int total = bars.size();
  int no = 0;
  
  // then loop through each bar
  Glib::ustring tip;
	
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i) {
    Bar &bar = **i;

    if (tip.empty())
      tip = bar.get_tip();
    else
      // note to translators: this is used for composing a list of monitors; %1
      // is the previous part of the list and %2 is the part to append
      tip = String::ucompose(_("%1\n%2"), tip, bar.get_tip());

    bar.update(*canvas, client, width(), height(), no++, total);
  }
  
  tooltips.set_tip(*canvas, tip);
}

void BarView::do_attach(Monitor *monitor, const Glib::ustring &mon_dir)
{
  bars.push_back(new Bar(monitor, mon_dir));
}

void BarView::do_detach(Monitor *monitor)
{
  for (bar_iterator i = bars.begin(), end = bars.end(); i != end; ++i)
    if ((*i)->get_monitor() == monitor) {
      delete *i;
      bars.erase(i);
      return;
    }

  g_assert_not_reached();
}

int BarView::width() const
{
  if (applet->horizontal())
    return size;
  else
    return applet->get_size();
}

int BarView::height() const
{
  if (applet->horizontal())
    return applet->get_size();
  else
    return size;
}

void BarView::resize_canvas()
{
  int w = width(), h = height();
  
  canvas->set_scroll_region(0, 0, w, h);
  canvas->set_size_request(w, h);
}
